<?php
/****************************************************************************
*    Copyright (C) 2000 Edikon Corporation.
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
****************************************************************************
*
* CLASS DESCRIPTION
*                   
* ps_cart
*
* The cart class is used to store products and carry them through the user's
* session in the store.
* propeties:  
* 	item() - an array of items
*       idx - the current count of items in the cart
*       error - the error message returned by validation if any
* methods:
*       add()
*       update()
*       delete()
*	
*
*************************************************************************/
class ps_cart {
  var $classname="ps_cart";
  var $persistent_slots = array("item", "idx");
  var $error;
  var $item = array();
  var $idx=0;


  /**************************************************************************
  ** name: add()
  ** created by: pablo
  ** description: adds an item to the shopping cart
  ** parameters:
  ** returns:
  ***************************************************************************/  
  function add(&$d) {
    
    $product_id = $d["product_id"];
    $quantity = $d["quantity"];

    // Quick add of item
    if ($d["product_id"]=="0") {
      $d["error"] = "Please select a service.";
      return False;
    } 

    // If no quantity sent them assume 1
    if ($quantity == "")
      $quantity = 1;
    // Check to see if we already have it
    $updated = 0;

    // Check for duplicate and add to current quantity
    for ($i=0;$i<$this->idx;$i++) {
      if ($this->item[$i]["product_id"] == $product_id) {
	$updated = 1;
      }
    }
    // If we did not update then add the item
    if (!$updated) {
      $this->item[$this->idx]["quantity"] = $quantity;
      $this->item[$this->idx]["product_id"] = $product_id;
      $this->idx++;
    }
    return True; 
  }

 /**************************************************************************
  ** name: update()
  ** created by: pablo
  ** description: updates the quantity of a product_id in the cart
  ** parameters:
  ** returns:
  ***************************************************************************/    
  function update($d) {
   
    $product_id = $d["product_id"];
    $quantity = $d["quantity"];

    if (!$product_id) {
      return False;
    }

    if ($quantity == 0) {
	$this->delete($d);
    }
    else {
      for ($i=0;$i<$this->idx;$i++) {
        if ($this->item[$i]["product_id"] == $product_id) {
	  $this->item[$i]["quantity"] = $quantity;
        }
      }
    }
    return True;
  }
  
 /**************************************************************************
  ** name: delete()
  ** created by: pablo
  ** description: deletes a given product_id from the cart
  ** parameters:
  ** returns:
  ***************************************************************************/    
  function delete($d) {
   
    $product_id = $d["product_id"];

    if (!$product_id) {
      return False;
    }
 
    $j = 0;
    for ($i=0;$i<$this->idx;$i++) {
      if ($this->item[$i]["product_id"] != $product_id) {
	$temp[$j++] = $this->item[$i];
      }
    }
    $this->idx--;
    $this->item = $temp;
    return True;
  } 


 /**************************************************************************
  ** name: reset()
  ** created by: pablo
  ** description: resets the cart (i.e. empty)
  ** parameters:
  ** returns:
  ***************************************************************************/    
  function reset() {
    $this->idx=0;
    return True;
  }
}
?>
