<?php
/****************************************************************************
*    Copyright (C) 2000 Edikon Corporation.
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
****************************************************************************
*
* CLASS DESCRIPTION
*                   
* ps_order_process
*
* The class contains the shop checkout code.  It is used to checkout
* and order and collect payment information.
* 
* propeties:  
*       class_name: Holds the name of the class.  Necessary for PHPLIB.
*               id: Another PHPLIB required variable.
*            error: Holds most recent error message set in by a method.
*
* methods:
*    validate_form:  Called to validate the values in checkout_form.  
*     validate_add:  Validates the checkout values prior to adding
*  validate_update:  Validate the checkout values prior to updating
*  validate_delete:  Validate form values prior to delete
*           update:  Update the order in the database
*           delete:  Delete the order in the database
*             find:  Find the order in the database
* ship_to_addresses_radio:  Get all the user_info Ship To (ST) records 
*                    associated with the $user_id and print an HTML 
*                    radio check box form element using the retrieved data.
*  display_address:  Print an HTML table displaying the user_info record
*                    for the specified $user_info_id and $address_type.
*              add:  Store the order information in the database
* get_order_number:  Create an order number using the session id, session
*                    name, and the current unix timestamp.
* calc_order_subtotal:  Calculate the order subtotal for the current order.  
*                    Does not include tax or shipping charges.
*   calc_order_tax:  Calculate the tax charges for the current order.
* calc_order_shipping:  Calculate the shipping charges for the current order
* calc_order_shipping_tax:  Calculate the tax for the shipping of the 
*                    current order
* get_vendor_currency:  Get the currency type used by the $vendor_id
*    email_receipt:  Create a receipt for the current order and email it to 
*                    the customer and the vendor.
* payment_method_type_long_name_display:  Print the full name of the 
*                    payment method type based off of the $short_name
*     asterisk_pad:  Return $str with all but $display_length at the end as 
*                    asterisks.
* dropdown_display:  Print an HTML dropdown box named $name using $arr to
*                    load the drop down.  If $value is in $arr, then $value
*                    will be the selected option in the dropdown.
*      validate_cc:  Validates credit card number format.
*    to_char_array:  Helper method for validate_cc()
*************************************************************************/
class ps_order_process {
  var $classname = "ps_order_process";
  var $id = "";
  var $error;

  /**************************************************************************
  ** name: validate_form()
  ** created by: gday
  ** description: Called to validate the form values in checkout_form
  ** parameters: $d 
  ** returns: True  - validation passed
  **          False - validation failed
  ***************************************************************************/
   function validate_form(&$d) {

   if (!$d["ship_to_info_id"]) {
     $d["error"] = "Please select a ship to address.";
     return False;
   }

   if ($d["ship_method_id"] == "0") {
     $d["error"] = "Please select a shipping method.";
     return False;
   }

   if (!$d["payment_method_type"]) {
     $d["error"] = "Please select a payment method.";
     return False;
   }
   
   if ($d["payment_method_type"] == "PO" && !$d["po_number"]) {
      $d["error"] = "Please enter a purchase order number.";
      return False;
   }

   if (!$d["payment_method_number"] && ($d["payment_method_type"] != "PO")) {
     $d["error"] = "Please enter a credit card number.";
     return False;
   } 

   if (!$d["payment_method_name"] && ($d["payment_method_type"] != "PO")) {
     $d["error"] = "Please enter the name on the credit card account.";
     return False;
   }

   if(strcasecmp($d["payment_method_type"], "PO") != 0 &&
      !$this->validate_cc($d["payment_method_number"], 
                   $d["payment_method_type"])) {
     $d["error"] = "The credit card account number entered is not valid.";
     return False;
   }

   if(!$d["payment_method_expire_month"] &&
      strcasecmp($d["payment_method_type"], "PO") != 0) {
     $d["error"] = "Please enter the credit card expiration month.";
     return False;
   }

   if(!$d["payment_method_expire_year"] &&
      strcasecmp($d["payment_method_type"], "PO") != 0) {
     $d["error"] = "Please enter the credit card expiration year.";
     return False;
   }
   $date = getdate(time());
   if ($d["payment_method_type"] != "PO" and ( 
       $d["payment_method_expire_year"] < $date["year"] or
      ($d["payment_method_expire_year"] == $date["year"] and
       $d["payment_method_expire_month"] < $date["mon"]))) {
     $d["error"] = "The expiration date is invalid.";
     return False;
   }

   // calculate the unix timestamp for the specified expiration date
   // default the day to the 1st

   $expire_timestamp = mktime(0,0,0,$d["payment_method_expire_month"],
                              1,$d["payment_method_expire_year"]);
   $d["payment_method_expire"] = $expire_timestamp;
   
   return True;
  }


  /**************************************************************************
  ** name: validate_add()
  ** created by: gday
  ** description:  Validates the checkout form values prior to adding
  ** parameters: $d 
  ** returns:  True - validation passed
  **          False - validation failed
  ***************************************************************************/
  function validate_add(&$d) {

   if (!$d["ship_to_info_id"]) {
     $this->error = "Please select a ship to address.";
     return False;
   }

   if (!$d["payment_method_type"]) {
     $this->error = "Please select a payment method.";
     return False;
   }

   if (!$d["payment_method_number"]) {
     $this->error = "Please enter a credit card number.";
     return False;
   }

   if(strcasecmp($d["payment_method_type"], "PO") != 0 &&
      !$this->validate_cc($d["payment_method_number"], 
                   $d["payment_method_type"])) {
     $d["error"] = "Invalid credit card number.";
     return False;
   }

   if(!$d["payment_method_expire"] &&
      strcasecmp($d["payment_method_type"], "PO") != 0) {
     $this->error = "Please enter the credit card expiration month.";
     return False;
   }
    
   return True;
  }

  /**************************************************************************
  ** name: validate_update()
  ** created by: gday
  ** description:  Validate the checkout values prior to updating
  ** parameters: $d
  ** returns:  True - validation passed
  **          False - validation failed
  ***************************************************************************/
  function validate_update($d) {
    
    return True;
  }
 
  /**************************************************************************
  ** name: validate_delete()
  ** created by: gday
  ** description:  Validate form values prior to delete
  ** parameters: $d
  ** returns:  True - validation passed
  **          False - validation failed
  ***************************************************************************/
  function validate_delete($d) {

    if(!$d["order_id"]) { 
       $this->error = "Unable to delete without the order id.";
       return False;
    }

    return True;
  }

  /**************************************************************************
  ** name: update()
  ** created by: gday
  ** description:  Update the order in the database
  ** parameters: $d
  ** returns:  True - update succeeded
  **          False - update failed
  ***************************************************************************/
  function update(&$d) {
    $db = new ps_DB;
    $timestamp = time();
    
    
    if ($this->validate_update($d)) {
      return True;
    }
    else {
      $d["error"] = $this->error;
      return False;
    }
  }
  
  /**************************************************************************
  ** name: delete()
  ** created by: gday
  ** description:  Delete the order in the database
  ** parameters: $d
  ** returns:  True - delete succeeded
  **          False - delete failed
  ***************************************************************************/
  function delete(&$d) {
    $db = new ps_DB;
    
    if ($this->validate_delete($d)) {
      $q = "DELETE from orders where order_id=" . $d["order_id"];
      $db->query($q);
      $db->next_record();

      $q = "DELETE from order_item where order_id=" . $d["order_id"];
      $db->query($q);
      $db->next_record();

      $q = "DELETE from payment_method where order_id=" . $d["order_id"];
      $db->query($q);
      $db->next_record();

      return True;
    }
    else {
      $d["error"] = $this->error;
      return False;
    }
  }
  

  /**************************************************************************
  ** name: find()
  ** created by: gday
  ** description:  Find the order in the database
  ** parameters: $d
  **             $start - where to begin displaying in the result set
  ** returns:
  ***************************************************************************/
  function find($d, $start=0) {
    
     return True;
  }


 /**************************************************************************
  ** name: ship_to_address_radio()
  ** created by: gday
  ** description:  Get all the user_info Ship To (ST) records associated
  **               with the $user_id and print an HTML radio check box
  **               form element using the retrieved data.
  ** parameters: $user_id - user id of to display ship to addresses
  **             $name - name of the HTML radio element
  **             $value - If matched, then this radio item will be 
  **                      checked
  ** returns:  Prints html radio element to standard out
  ***************************************************************************/
   function ship_to_addresses_radio($user_id, $name, $value) {
      global $sess;

      $db = new ps_DB;
          
      /* Select all the ship to information for this user id and
       * order by modification date; most recently changed to oldest
       */

      $q  = "SELECT user_info_id FROM user_info ";
      $q .= "WHERE user_id = '" . $user_id . "' ";
      $q .= "AND address_type = 'BT' ";
      $db->query($q);
      $db->next_record();
      $bt_user_info_id = $db->f("user_info_id");
 
      $q  = "SELECT user_info_id, address_type_name, company, title, ";
      $q .= "last_name, first_name, middle_name, phone_1, phone_2, ";
      $q .= "fax, address_1, address_2, city, ";
      $q .= "state, country, zip ";
      $q .= "FROM user_info ";
      $q .= "WHERE user_id = '" . $user_id . "' ";
      $q .= "AND address_type = 'ST' ";
      $q .= "ORDER by address_type_name, mdate DESC";

      $db->query($q);

      echo "<TABLE BORDER=0 WIDTH=100% CELLPADDING=2 CELLSPACING=0>\n";
      echo "<TR>\n";
      echo "<TD>\n";
      if ($db->num_rows() and $bt_user_info_id != $value) {
        echo "<INPUT TYPE=radio NAME=$name VALUE=\"$bt_user_info_id\">\n";
      } else {
        echo "<INPUT TYPE=radio NAME=$name VALUE=\"$bt_user_info_id\" CHECKED>\n";
      }
      echo "</TD>\n";
      echo "<TD bgcolor=#f0f0f0>\n";
      echo "<font face=arial,helvetica size=\"-1\"><b>";
      echo "Same as Billing Address.</b></font>\n";
      echo "</TD>\n";
      echo "</TR>\n";
      while($db->next_record()) {
         echo "<TR>\n";
         echo "<TD>\n";
         if (!strcmp($value, $db->f("user_info_id"))) {
            echo "<INPUT TYPE=radio NAME=$name VALUE=" . $db->f("user_info_id") . " CHECKED>\n";
         }
         else {
            echo "<INPUT TYPE=radio NAME=$name VALUE=" . $db->f("user_info_id") . ">\n";
         }
         echo "</TD>\n";
         echo "<TD>\n";
         echo "<TABLE width=100% border=0 cellspacing=0 cellpadding=1 bgcolor=#f0f0f0>\n";
         echo "<TR>\n";
         echo "<TD>\n";
         echo "<font face=arial,helvetica size=\"-1\">\n";
         echo "<b>" . $db->f("address_type_name") . "</b> ";
         $url = SECUREURL . "?page=order/checkout_address_form&amp;user_info_id=" . $db->f('user_info_id');
         echo "(<a href=\"";
         $sess->purl($url);
         echo "\">Update Address</a>)\n";
         echo "<BR>\n";
         echo $db->f("title") . " ";
         echo $db->f("first_name") . " ";
         echo $db->f("middle_name") . " ";
         echo $db->f("last_name") . "\n";
         echo "<BR>\n";
         if ($db->f("company")) {
           echo $db->f("company") . "\n";
           echo "<BR>\n";
         }
         echo $db->f("address_1") . "\n";
         if ($db->f("address_2")) {
            echo "<BR>";
            echo $db->f("address_2"). "\n";
         }
         echo "<BR>\n";
         echo $db->f("city");
         echo ", ";
         echo $db->f("state") . " ";
         echo $db->f("zip") . "\n";
         echo "<BR>\n";
         echo "Phone:";
         echo $db->f("phone_1") . "\n";
         echo "<BR>\n";
         echo "Fax:";
         echo $db->f("fax") . "\n";
         echo "</FONT>\n";
         echo "</TD>\n";
         echo "</TR>\n";
         echo "</TABLE>\n";
      }

      echo "</TD>\n";
      echo "</TR>\n";
      echo "</TABLE>\n";

      return(true);
   }

  /**************************************************************************
  ** name: display_address()
  ** created by: gday
  ** description:  Print an HTML table displaying the user_info record
  **               for the specified $user_info_id and $address_type.
  ** parameters: $user_info_id - user info id to display
  **             $address_type - address type (BT or ST)
  ** returns: Prints HTML table displaying the address information
  ***************************************************************************/
   function display_address($user_info_id) {
      $db = new ps_DB;

      $q = "SELECT address_type_name, company, title, last_name, ";
      $q .= "first_name, middle_name, phone_1, phone_2, fax, ";
      $q .= "address_1, address_2, city, state, country, zip ";
      $q .= "FROM user_info ";
      $q .= "WHERE user_info_id = '$user_info_id'";

      $db->query($q);

      if($db->next_record()) {
         echo "<TABLE width=100% border=0 cellspacing=0 cellpadding=1>";
         echo "<TR>";
         echo "<TD ALIGN=center>";
         echo "<font face=arial,helvetica size=\"-1\">";
         if ($db->f("address_type_name") != "-default-") {
           echo "<b>" . $db->f("address_type_name") . "</b><br>";
         }
         echo $db->f("title") . " ";
         echo $db->f("first_name") . " ";
         echo $db->f("middle_name") . " ";
         echo $db->f("last_name") . " ";
         echo "<BR>";
         if ($db->f("company")) {
           echo $db->f("company");
           echo "<BR>";
         }
         echo $db->f("address_1");
         if ($db->f("address_2")) {
            echo "<BR>";
            echo $db->f("address_2");
         }
         echo "<BR>";
         echo $db->f("city");
         echo ", ";
         echo $db->f("state") . " ";
         echo $db->f("zip");
         echo "<BR>";
         echo "Phone:";
         echo $db->f("phone_1");
         echo "<BR>";
         echo "Fax:";
         echo $db->f("fax");
         echo "</FONT>";
         echo "</TD>";
         echo "</TR>";
         echo "</TABLE>";
      }

      return True;
   }

  /**************************************************************************
  ** name: add()
  ** created by: gday
  ** description:  Store the order information in the database
  ** parameters: $d
  ** returns:  True - order information stored
  **          False - Failure in storing the order information
  ***************************************************************************/
   function add(&$d) {
      global $auth,
             $HTTP_POST_VARS,
	     $ps_cart,
	     $ps_vendor_id;
      $db = new ps_DB;
  

      if ($this->validate_add($d)) {
         $order_number = $this->get_order_number();
         $order_subtotal = $this->calc_order_subtotal($d);
         $order_tax = $this->calc_order_tax($order_subtotal, $d);
         $order_shipping = $this->calc_order_shipping($d);
         $order_shipping_tax = $this->calc_order_shipping_tax($d);

         $timestamp = time();

         /* Insert the main order information */
         $q = "INSERT INTO orders ";
         $q .= "(user_id, vendor_id, order_number, user_info_id, ";
         $q .= "ship_method_id, order_subtotal, order_tax, order_shipping, ";
         $q .= "order_shipping_tax, order_currency, order_status, cdate, ";
         $q .= "mdate) ";
         $q .= "VALUES (";
         $q .= "'" . $auth->auth["uid"] . "', ";
         $q .= $ps_vendor_id . ", ";  
         $q .= "'" . $order_number . "', ";
         $q .= $d["ship_to_info_id"] . ", '";
         $q .= $d["ship_method_id"] . "', ";
         $q .= $order_subtotal . ", ";
         $q .= $order_tax . ", "; 
         $q .= $order_shipping . ", ";
         $q .= $order_shipping_tax . ", ";
         $q .= "'', "; /* Currency is at the product level - line item */
         $q .= "'P', ";
         $q .= $timestamp . ", ";
         $q .= $timestamp;
         $q .= ")";

         $db->query($q);
         $db->next_record();

         /* Get the order id just stored */

         $q = "SELECT order_id FROM orders WHERE order_number = ";
         $q .= "'" . $order_number . "'";

         $db->query($q);
         $db->next_record();

         $order_id = $db->f("order_id");

         /* Insert the Order payment info */

         $payment_number = ereg_replace(" |-", "", $d["payment_method_number"]);

         /* Need to encrypt payment_number before storing it in the db. */
          if (MCRYPT) {
            $payment_number = mcrypt_ecb(MCRYPT_TripleDES,
                                        SECRET_KEY,
                                        $payment_number,
                                        MCRYPT_ENCRYPT);
          }

         $q = "INSERT INTO payment_method ";
         $q .= "(order_id, payment_method_type, payment_method_number, ";
         $q .= "payment_method_expire, payment_method_name) ";
         $q .= "VALUES (";
         $q .= $order_id . ", ";
         $q .= "'" . $d["payment_method_type"] . "', ";
         $q .= "'" . $payment_number . "', ";
         $q .= "'" . $d["payment_method_expire"] . "',";
         $q .= "'" . $d["payment_method_name"] . "'";
         $q .= ")";

         $db->query($q);
         $db->next_record();

         /* Insert the order line items; one row per product in the cart */

         for($i = 0; $i < $ps_cart->idx; $i++) {

            $product_price_arr = get_price($ps_cart->item[$i]["product_id"]);

            $product_price = $product_price_arr["product_price"];

            $vendor_id = $ps_vendor_id;

            $product_currency = $product_price_arr["product_currency"];

            $q = "INSERT INTO order_item ";
            $q .= "(order_id, user_info_id, vendor_id, ";
            $q .= "product_id, product_quantity, product_item_price, ";
            $q .= "order_item_currency, order_status, cdate, mdate) ";
            $q .= "VALUES ('";
            $q .= $order_id . "', '";
            $q .= $d["ship_to_info_id"] . "', '";
            $q .= $vendor_id . "', '";
            $q .= $ps_cart->item[$i]["product_id"] . "', '";
            $q .= $ps_cart->item[$i]["quantity"] . "', '";
            $q .= $product_price . "', '";
            $q .= $product_currency . "', ";
            $q .= "'P','";
            $q .= $timestamp . "','";
            $q .= $timestamp . "'";
            $q .= ")";

            $db->query($q);
            $db->next_record();
         } 

         // Export the order_id so the checkout complete page can get it
         $d["order_id"] = $order_id;

         // Unset the payment_method variables 

         $d["payment_method_type"] = "";
         $d["payment_method_number"] = "";
         $d["payment_method_expire"] = "";
         $d["payment_method_name"] = "";
       
         $HTTP_POST_VARS["payment_method_type"] = "";
         $HTTP_POST_VARS["payment_method_number"] = "";         
         $HTTP_POST_VARS["payment_method_expire"] = "";
         $HTTP_POST_VARS["payment_method_name"] = "";

         // Reset the cart 
	 $ps_cart->reset();

         // Send the e-mail confirmation messages
         $this->email_receipt($order_id);
      }
      else {

         $d["error"] = $this->error;
         return False;
      }

      return True;
   }

  /**************************************************************************
  ** name: get_order_number()
  ** created by: gday
  ** description:  Create an order number using the session id, session
  **               name, and the current unix timestamp.
  ** parameters: 
  ** returns: unique order_number
  ***************************************************************************/
   function get_order_number() {
      global $sess;

      /* Generated a unique order number */

      $str = (string)$sess->id;
      $str .= (string)$sess->name;
      $str .= (string)time();

      $order_number = md5($str);

   return($order_number);
   }


  /**************************************************************************
  ** name: calc_order_subtotal()
  ** created by: gday
  ** description:  Calculate the order subtotal for the current order.  
  **               Does not include tax or shipping charges.
  ** parameters: $d
  ** returns: sub total for this order
  ***************************************************************************/
   function calc_order_subtotal($d) {
      global $auth, $ps_cart;
      $subtotal = 0.0;
     
      $db = new ps_DB;
 
      for($i = 0; $i < $ps_cart->idx; $i++) {
            $product_price_arr = get_price($ps_cart->item[$i]["product_id"]);
            $subtotal += $product_price_arr["product_price"] * $ps_cart->item[$i]["quantity"];
      }         

      return($subtotal); 
   }

  /**************************************************************************
  ** name: calc_order_tax()
  ** created by: gday
  ** description:  Calculate the tax charges for the current order.
  ** parameters: $d
  **             $order_subtotal - sub total for the order
  ** returns: Tax for the current order
  ***************************************************************************/
   function calc_order_tax($order_subtotal, $d) {

      /* 01/24/2000 - Dummy function for now */

      return(0);
   }

  /**************************************************************************
  ** name: calc_order_shipping()
  ** created by: gday
  ** description:  Calculate the shipping charges for the current order
  ** parameters: $d
  ** returns: Shipping costs for this order
  ***************************************************************************/
   function calc_order_shipping($d) {

      /* 01/25/2000 - Dummy function for now */

      return(0);
   }
      
  /**************************************************************************
  ** name: calc_order_shipping_tax()
  ** created by: gday
  ** description:  Calculate the tax for the shipping of the current order
  ** parameters: $d
  ** returns: Tax for the shipping of this order
  ***************************************************************************/
   function calc_order_shipping_tax($d) { 

      /* 01/25/2000 - Dummy function for now */

      return(0);
   }
    
  /**************************************************************************
  ** name: get_vendor_currency()
  ** created by: gday
  ** description:  Get the currency type used by the $vendor_id
  ** parameters: $vendor_id - vendor id to return currency type
  ** returns: Currency type for this vendor
  ***************************************************************************/
   function get_vendor_currency($vendor_id) {
      $db = new ps_DB;

      $q = "SELECT vendor_currency FROM vendor WHERE vendor_id = " . $vendor_id;

      $db->query($q);
      $db->next_record();

      $currency = $db->f("vendor_currency");

      return($currency);
   }

  /**************************************************************************
  ** name: email_receipt()
  ** created by: gday
  ** description:  Create a receipt for the current order and email it to 
  **               the customer and the vendor.
  ** parameters: $order_id - Order ID for which to create the email receipts
  ** returns:  True - receipt created and emailed
  **          False - error occured 
  ***************************************************************************/
   function email_receipt($order_id) {
       global $sess,
              $ps_product,
              $ps_vendor_id;

       $db = new ps_DB;
       $q  = "SELECT * FROM orders,ship_method ";
       $q .= "WHERE order_id='$order_id'";
       $q .= "AND orders.ship_method_id = ship_method.ship_method_id";
       $db->query($q);
       $db->next_record();

       $user_id = $db->f("user_id");

       $dbbt = new ps_DB;
       $qt = "SELECT * from auth_user_md5, user_info ";
       $qt .= "WHERE user_info.user_id='$user_id' ";
       $qt .= "AND user_info.address_type='BT'";
       $dbbt->query($qt);
       $dbbt->next_record();

       $dbst = new ps_DB;
       $qt = "SELECT * FROM user_info ";
       $qt .= "WHERE user_info_id='";
       $qt .= $db->f("user_info_id") . "'";
       $dbst->query($qt);
       $dbst->next_record(); 

       $dbpm = new ps_DB;
       $qt = "SELECT * from payment_method ";
       $qt .= "WHERE order_id='$order_id'";
       $dbpm->query($qt);
       $dbpm->next_record();
   
       $dbv = new ps_DB;
       $qt = "SELECT * from vendor ";
/* Need to decide on vendor_id <=> order relationship */

       $qt .= "WHERE vendor_id = $ps_vendor_id";
       $dbv->query($qt);
       $dbv->next_record();

       $shopper_header = "Thank you for shopping with us.  Your order ";
       $shopper_header .= "information follows.\n\n";

       $shopper_footer = "\n\nThank you for your patronage.\n";
       $shopper_footer .= "\n\nQuestions? Problems?\n";
       $shopper_footer .= "Email: " . $dbv->f("contact_email");
       
       $shopper_subject = $dbv->f("vendor_name") . " Order -" . $db->f("order_id");

       $vendor_header = "The following order was received.\n";
       $vendor_subject = "Order Notice-" . $db->f("order_id");

       $url = $sess->url(SECUREURL . "?page=admin/order_display&amp;order_id=$order_id");

       $vendor_footer = "\n\nView the order by following the link below.\n";
       $vendor_footer .= $sess->url($url);

       $shopper_email = $dbbt->f("user_email");
       $vendor_email = $dbv->f("contact_email");
 
       $shopper_message  = "\nPURCHASE ORDER\n";
       $shopper_message .= "--------------\n";
       $shopper_message .= "ORDER NUMBER: " . $db->f("order_id") . "\n";
       $shopper_message .= "ORDER DATE:   ";
       $shopper_message .= date("d-M-Y:H:i", $db->f("cdate")) . "\n";
       $shopper_message .= "ORDER STATUS: ";
       switch($db->f("order_status")) {
         case ("P"):
           $shopper_message .= "Pending\n\n";
           break;
         case ("X"):
           $shopper_message .= "Canceled\n\n";
           break;
         case ("C"):
           $shopper_message .= "Confirmed\n\n";
           break;
       }

       $shopper_message .= "CUSTOMER INFORMATION\n";
       $shopper_message .= "--------------------\n\n";
       $shopper_message .= "Bill To\n";
       $shopper_message .= "-------\n\n";
       $shopper_message .= "     Company:    ";
       $shopper_message .= $dbbt->f("company") . "\n";
       $shopper_message .= "     Name:       ";
       if ($dbbt->f("title")) {
         $shopper_message .= $dbbt->f("title") . " ";
       }
       $shopper_message .= $dbbt->f("first_name") . " ";
       if ($dbbt->f("middle_name")) {
         $shopper_message .= $dbbt->f("middle_name") . " ";
       }
       $shopper_message .= $dbbt->f("last_name") . "\n";
       $shopper_message .= "     Address1:   ";
       $shopper_message .= $dbbt->f("address_1") . "\n";
       $shopper_message .= "     Address2:   ";
       $shopper_message .= $dbbt->f("address_2") . "\n";
       $shopper_message .= "     City:       ";
       $shopper_message .= $dbbt->f("city") . "\n";
       $shopper_message .= "     State:      ";
       $shopper_message .= $dbbt->f("state") . "\n";
       $shopper_message .= "     Zip:        ";
       $shopper_message .= $dbbt->f("zip") . "\n";
       $shopper_message .= "     Country:    ";
       $shopper_message .= $dbbt->f("country") . "\n";
       $shopper_message .= "     Phone:      ";
       $shopper_message .= $dbbt->f("phone_1") . "\n";
       $shopper_message .= "     Fax:        ";
       $shopper_message .= $dbbt->f("fax") . "\n\n";
       
       $shopper_message .= "Ship To\n";
       $shopper_message .= "-------\n\n";
       $shopper_message .= "     Company:    ";
       $shopper_message .= $dbst->f("company") . "\n";
       $shopper_message .= "     Name:       ";
       $shopper_message .= $dbbt->f("title") . " ";
       $shopper_message .= $dbst->f("first_name") . " ";
       $shopper_message .= $dbst->f("middle_name") . " ";
       $shopper_message .= $dbst->f("last_name") . "\n";
       $shopper_message .= "     Address1:   ";
       $shopper_message .= $dbst->f("address_1") . "\n";
       $shopper_message .= "     Address2:   ";
       $shopper_message .= $dbst->f("address_2") . "\n";
       $shopper_message .= "     City:       ";
       $shopper_message .= $dbst->f("city") . "\n";
       $shopper_message .= "     State:      ";
       $shopper_message .= $dbst->f("state") . "\n";
       $shopper_message .= "     Zip:        ";
       $shopper_message .= $dbst->f("zip") . "\n";
       $shopper_message .= "     Country:    ";
       $shopper_message .= $dbst->f("country") . "\n";
       $shopper_message .= "     Phone:      ";
       $shopper_message .= $dbst->f("phone_1") . "\n";
       $shopper_message .= "     Fax:        ";
       $shopper_message .= $dbst->f("fax") . "\n\n";

       $shopper_message .= "ORDER ITEMS\n";
       $shopper_message .= "-----------";

       $dboi = new ps_DB;
       $q = "SELECT * ";
       $q .= "FROM product, order_item, orders ";
       $q .= "WHERE product.product_id=order_item.product_id ";
       $q .= "AND order_item.order_id='";
       $q .= $order_id . "' ";
       $q .= "AND orders.order_id=order_item.order_id";
       $dboi->query($q);

       while($dboi->next_record()) {
          $shopper_message .= "\n\n";
          $shopper_message .= "PRODUCT  = ";
          if ($dboi->f("product_parent_id")) {
            $shopper_message .= $ps_product->get_field($dboi->f("product_parent_id"), "product_name") . "\n";
            $shopper_message .= "SERVICE  = ";
          }
          $shopper_message .= $dboi->f("product_name") . "\n";
          $shopper_message .= "QUANTITY = ";
          $shopper_message .= $dboi->f("product_quantity") . "\n";
          $shopper_message .= "SKU      = ";
          $shopper_message .= $ps_product->get_field($dboi->f("product_id"), "product_sku") . "\n";
          $price = get_price($dboi->f("product_id"));
          $shopper_message .= "PRICE    = ";
          $shopper_message .= sprintf("%1.2f %s", $price["product_price"], $price["product_currency"]);
       }

       $shopper_message .= "\n\n";
       $sub_total = $db->f("order_subtotal");
       $shopper_message .= "SUBTOTAL = ";
       $shopper_message .= sprintf("%1.2f %s\n", $sub_total, $price["product_currency"]);
       $order_tax = $db->f("order_tax");
       $shopper_message .= "TAX      = ";
       if ($dbbt->f("extra_field_5") == "Y") {
         $shopper_message .= "Reseller";
       } else {
         $shopper_message .= "Taxes apply for GA, CA, WA, TX, and FL.";
       }
       //$shopper_message .= sprintf("%1.2f", $order_tax);

       /*
       $order_shipping = $db->f("order_shipping") . "\n";
       $shopper_message .= "SHIPPING = ";
       $shopper_message .= sprintf("%1.2f", $order_shipping);
       */

       // Commented Out
       /* 
       $order_shipping_tax = $db->f("order_shipping_tax") . "\n";
       $shopper_message .= "SHIPPING TAX   = ";
       $shopper_message .= sprintf("%1.2f", $order_shipping_tax);
       */

       $order_total= $sub_total + 
                     $order_tax + 
                     $order_shipping + 
                     $order_shipping_tax;
       $shopper_message .= "\n\n";
       $shopper_message .= "TOTAL    = ";
       $shopper_message .= sprintf("%1.2f %s", $order_total, $price["product_currency"]);


       // Shipping Method       
       $shopper_message .= "\n\n";
       $shopper_message .= "Shipping Method\n";
       $shopper_message .= "---------------\n";
       $shopper_message .= "PREFERRED CARRIER = ";
       $shopper_message .= $db->f("ship_carrier") . "\n";
       $shopper_message .= "SHIPPING MODE     = ";
       $shopper_message .= $db->f("ship_mode");

       // Payment Method       
       $shopper_message .= "\n\n";
       $shopper_message .= "Payment Method\n";
       $shopper_message .= "--------------\n";
      
       $shopper_message .= "PAYMENT METHOD   = ";
       $shopper_message .= $dbpm->f("payment_method_type") . "\n";
       if ($dbpm->f("payment_method_type") != "PO") {
         $shopper_message .= "ACCOUNT NAME     = ";
         $shopper_message .= $dbpm->f("payment_method_name") . "\n";
       }
       $shopper_message .= "ACCOUNT NUMBER   = ";

       $payment_number = $dbpm->f("payment_method_number");

       /* Need to encrypt payment_number before storing it in the db.
        *
        * $payment_number = mcrypt_ecb(MCRYPT_TripleDES, 
        *                              SECRET_KEY, 
        *                              $payment_number, 
        *                              MCRYPT_DECRYPT);
        */

       if ($dbpm->f("payment_method_type") =="PO") {
          $shopper_message .= $payment_number;
       }
       else {
          $shopper_message .= $this->asterisk_pad($payment_number, 4) . "\n"; 
          $shopper_message .= "EXPIRE DATE      = ";
          $shopper_message .= date("m/Y", $dbpm->f("payment_method_expire"));
       }
    

       // Mail receipt to the shopper
       mail($shopper_email, 
            $shopper_subject, 
            $shopper_header . $shopper_message . $shopper_footer,
            "From: phpshop@edikon.com\n");

       // Mail receipt to the vendor
       mail($vendor_email,
            $vendor_subject,
            $vendor_header . $shopper_message . $vendor_footer,
            "From: phpshop@edikon.com\n");

      return(True); 
   }

  /**************************************************************************
  ** name: payment_method_type_long_name_display()
  ** created by: gday
  ** description:  Print the full name of the payment method type based off
  **               of the $short_name
  ** parameters: $short_name - short code for the payment method name
  ** returns: Full name of the payment method
  ***************************************************************************/
  function payment_method_type_long_name_display($short_name) {
  
      $payment_type = array("MC"=>"Master Card",
                           "VISA"=>"Visa", 
                           "AMEX"=>"American Express",
                           "DISCOVER"=>"Discover",
                           "PO"=>"Purchase Order");

      if(!empty($payment_type[$short_name])) {
         echo $payment_type[$short_name];
      }
      else {
         echo $short_name;
      }
    
      return True;
  }

  /**************************************************************************
  ** name: asterisk_pad()
  ** created by: gday
  ** description:  Return $str with all but $display_length at the end as 
  **               asterisks.
  ** parameters: $str - string to asterisk pad
  **             $display_length - length at the end of $str that will not
  **                               be obscured by asterisks
  ** returns: $str with all but $display_length at the end obscured
  **          by asterisks
  ***************************************************************************/
   function asterisk_pad($str, $display_length) {

      $total_length = strlen($str);

      if($total_length > $display_length) {
         for($i = 0; $i < $total_length - $display_length; $i++) {
           $str[$i] = "*";
         }
      }

      return($str);
   } 

  /**************************************************************************
  ** name: dropdown_display()
  ** created by: gday
  ** description:  Print an HTML dropdown box named $name using $arr to
  **               load the drop down.  If $value is in $arr, then $value
  **               will be the selected option in the dropdown.
  ** parameters: $name - name of the HTML dropdown element
  **             $value - Drop down item to make selected
  **             $arr - array used to build the HTML drop down element
  ** returns: prints HTML drop down element to standard output
  ***************************************************************************/
   function dropdown_display($name, $value, $arr) {

      echo "<SELECT NAME=\"$name\" SIZE=\"1\">\n";

      while (list($key, $val) = each($arr)) {
         if(strcmp($value, $key) == 0) {
            echo "<OPTION VALUE=\"$key\" SELECTED>$val\n";
         }
         else {
            echo "<OPTION VALUE=\"$key\">$val\n";
         }
      }

      echo "</SELECT>\n";

      return True;
   }

  /**************************************************************************
  ** This code was adapted from validateCC() by Brett Error.
  **
  ** name: validate_cc()
  ** created by: gday
  ** description:  Validates credit card number format.  If $type is passed, 
  **               the function ensures that $ccnum follows the proper format
  **               for that credit card type.  Valid $types are unknown -
  **               type not known or not supported (default), mc - 
  **               Mastercard, visa - Visa, amex - American Express, and
  **               discover - Discover.  The function will also do a 
  **               mod 10 check to verify that it could be a valid 
  **               credit card number.  
  ** parameters: $ccnum - credit card number to validate
  **             $type - credit card type
  ** returns:  True - credit card number is in a valid format
  **          False - credit card number is not in a valid format
  ***************************************************************************/
   function validate_cc($ccnum, $type =  'unknown') { 
   global $sess; 
 
   //Clean up input 
   $type = strtolower($type); 
   $ccnum = ereg_replace( '[-[:space:]]',  '',$ccnum);  

   //Do type specific checks 

   if ($type ==  'unknown') { 

      //Skip type specific checks 

   } 
   elseif ($type ==  'mc'){ 
      if (strlen($ccnum) != 16 || !ereg( '^5[1-5]', $ccnum)) {
         return(false); 
      }
   } 
   elseif ($type ==  'visa'){ 
      if ((strlen($ccnum) != 13 && strlen($ccnum) != 16) || substr($ccnum, 0, 1) !=  '4') {
         return(false); 
      }
   } 
   elseif ($type ==  'amex'){ 
      if (strlen($ccnum) != 15 || !ereg( '^3[47]', $ccnum)) {
         return(false); 
      }
   } 
   elseif ($type ==  'discover'){ 
      if (strlen($ccnum) != 16 || substr($ccnum, 0, 4) !=  '6011') {
         return(false); 
      }
   } 
   else { 
      //invalid type entered 
      return(false); 
   } 


   // Start MOD 10 checks 

   $dig = $this->to_char_array($ccnum); 
   $numdig = sizeof ($dig); 
   $j = 0; 
   for ($i=($numdig-2); $i>=0; $i-=2){ 
      $dbl[$j] = $dig[$i] * 2; 
      $j++; 
   }     
   $dblsz = sizeof($dbl); 
   $validate =0; 
   for ($i=0;$i<$dblsz;$i++){ 
      $add = $this->to_char_array($dbl[$i]); 
      for ($j=0;$j<sizeof($add);$j++){ 
         $validate += $add[$j]; 
      } 
      $add =  ''; 
   } 
   for ($i=($numdig-1); $i>=0; $i-=2){ 
      $validate += $dig[$i];  
   } 
   if (substr($validate, -1, 1) ==  '0') {
      return(true); 
   }
   else {
      return(false); 
   }
   } // validate_cc

 /****************************************************************************
 * Copyright Information:
 *
 * This code was adapted from validateCC() by Brett Error.
 *  
 * Credit card validation routine                 
 * May 15, 1998                                  
 * By Brett Error                                
 * brett@interwebdesign.com                     
 *****************************************************************************/
   function to_char_array($input) { 

      $len = strlen($input); 

      for ($j=0;$j<$len;$j++){ 
         $char[$j] = substr($input, $j, 1);     
      } 
      return ($char); 
   } // to_char_array()

} 

?>
